{vs. 1.0  1/9/2015
Revilla, E. (in press). Individual and agent based models in population ecology and conservation biology.
In: Population Ecology in Practice: Underused, Misused, and Abused Methods.
Eds Murray DL, Sandercock B. John Wiley & Sons Ltd. ISBN/ISSN: 9780470674130}

unit density_dependent_unit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, TAGraph, TASeries, Forms, Controls, Graphics,
  Dialogs, StdCtrls, ExtCtrls, math;

type           {here you declare the data structure for you individuals}
  PAgent=^Agent;
  Agent = record
    sex : string[1];
    Age : integer;
    Status : Shortint;
  end;

  { Tdensity_dependent_Form }

  Tdensity_dependent_Form = class(TForm)
    Chart1: TChart;
    Chart1LineSeries1: TLineSeries;
    Chart1LineSeries2: TLineSeries;
    Chart2: TChart;
    Chart2LineSeries1: TLineSeries;
    Chart2LineSeries2: TLineSeries;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Edit6: TEdit;
    Edit7: TEdit;
    Edit8: TEdit;
    Edit9: TEdit;
    Exit_Button: TButton;
    Abort_Button: TButton;
    Edit1: TEdit;
    Image1: TImage;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Memo1: TMemo;
    Panel1: TPanel;
    Run_Button: TButton;
    procedure Abort_ButtonClick(Sender: TObject);
    procedure Exit_ButtonClick(Sender: TObject);
    procedure Run_ButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
    Procedure Pop_dynamics;
  end;

var               {here you declare global variables}
  density_dependent_Form: Tdensity_dependent_Form;
  Population : Tlist;
  Individual : PAgent;
  n_ini : integer;
  rep_prob: real;
  AlphaR:real;
  BetaR:real;
  surv_prob: real;
  AlphaS:real;
  BetaS:real;
  max_age:integer;
  populationsize: integer;
  males_available:string;
  n_sim: integer;
  n_extint:integer;
  current_sim:integer;
  max_pop_size: integer;
  pop_size: array[1..100] of integer;
  sum_pop_size: array[1..100] of integer;
  to_file_out: TextFile;

const             {here you declare constants}
  max_years = 100;
  min_rep_age = 3;
  file_name = 'output.txt';

implementation

{$R *.lfm}

{ Tdensity_dependent_Form }


Procedure Startpopulation;
var
a:integer;
begin
  Population:=Tlist.create;
  with population do
  begin
    for a:=1 to n_ini do
    begin
      new(Individual);
      Individual^.age:=random(10);        {alternative: Individual^.age:=0;}
      if random<0.5 then Individual^.sex:= 'f' else Individual^.sex:= 'm';
      Individual^.status:=0;     {0 for non breeding age; 1 for breeding age}
      Population.add(Individual);
    end;
  end;
end;

Procedure Reproduction;
var
a:integer;
current_rep:real;
begin
  with population do
  begin
    populationsize:=population.count;
    current_rep:=rep_prob/(1+AlphaR*power(populationsize,BetaR));
    if (populationsize>1) and (males_available='yes') then
    for a:=0 to populationsize-1 do
    begin
      Individual:=Items[a];
      if individual^.sex='f' then
      if individual^.status=1 then
      if random<current_rep then
      begin
        New(Individual);
        Individual^.age:=0;
        if random<0.5 then Individual^.sex:= 'f' else Individual^.sex:= 'm';
        Individual^.status:=0;
        Population.add(Individual);
      end;
    end;
  end;
end;


Procedure survival;
var
a,b:integer;
current_surv:real;
die:string;
begin
  {survival part}
  with population do
  begin
    b:=-1;
    populationsize:=population.count;

    current_surv:=surv_prob/(1+AlphaS*power(populationsize,BetaS));
    //current_surv:=randg(surv_prob, 0.1)/(1+AlphaS*power(populationsize,BetaS));
    for a:=0 to populationsize-1 do  {the index in the list starts at 0}
    begin
      b:=b+1;
      individual:=items[b]; {we need a new index b to be able to delete the individuals from the list}
      die:='no';
      if individual^.age>max_age then die:='yes' else
        if random>current_surv then die:='yes';
      if die='yes' then
      begin
        delete(b);
        dispose(individual);
        b:=b-1;           {to keep the index correct}
      end;
    end;
  end;

  {aging part + }
  populationsize:=population.count;
  males_available:='no';
  with population do
  begin
    for a:=0 to populationsize-1 do
    begin
      Individual:=Items[a];
      Individual^.age:=Individual^.age+1;
      if individual^.age>=min_rep_age then individual^.status:=1 else individual^.status:=0;
      if (individual^.sex='m') and (individual^.status=1) then males_available:='yes';
    end;
  end;

end;


Procedure Tdensity_dependent_Form.Pop_dynamics;
var
a:integer;

begin
  with population do
  begin
    for a:=1 to max_years do
    begin
      reproduction;   //call procedure for reproduction
      survival;       //call procedure for survival
      {ploting pop size}
      if max_pop_size<populationsize then max_pop_size:=populationsize;
      if max_pop_size>Chart1.extent.YMax then
      begin
        Chart1.extent.YMax:=max_pop_size;
        application.processmessages;
      end;
      pop_size[a]:=populationsize;
      sum_pop_size[a]:=sum_pop_size[a]+ populationsize;

      {plot trajectory}
      Chart1LineSeries1.addxy(a,populationsize);

    end;
    if  populationsize=0 then N_extint:=N_extint+1;
  end;
end;


procedure Tdensity_dependent_Form.Run_ButtonClick(Sender: TObject);
var
  a,b: integer;
  t:string;

begin
  randomize; {initialize the pseudorandom number generator}
  val(Edit1.text,n_ini);  {read parameter values from the form}
  val(Edit2.text,rep_prob);
  val(Edit3.text,surv_prob);
  val(Edit4.text,max_age);
  val(Edit5.text,n_sim);
  val(Edit6.text,AlphaS);
  val(Edit7.text,BetaS);
  val(Edit8.text,AlphaR);
  val(Edit9.text,BetaR);

  // plot density dependent rate responses
  Chart2LineSeries1.clear;
  Chart2LineSeries2.clear;
  for b:=1 to 100 do Chart2LineSeries1.addxy(b,rep_prob/(1+AlphaR*power(b,BetaR)));//reproduction
  for b:=1 to 100 do Chart2LineSeries2.addxy(b,surv_prob/(1+AlphaS*power(b,BetaS)));//survival

  N_extint:=0;

  AssignFile(to_file_out, file_name);
  rewrite(to_file_out); {create txt file}

  for a:=1 to 100 do sum_pop_size[a]:=0;
  for current_sim:=1 to n_sim do
  begin

    max_pop_size:=0;
    Startpopulation; {call the procedure to initialize your population}
    Pop_dynamics;    {call the procedure to run the population dynamics}


    {plot population trayectories}
    Chart1LineSeries1.clear;
    Chart1LineSeries2.clear;
    for b:=1 to 100 do Chart1LineSeries1.addxy(b,pop_size[b]);
    for b:=1 to 100 do Chart1LineSeries2.addxy(b,sum_pop_size[b]/current_sim);

    {extinction probability}
    if current_sim>0 then str(n_extint/current_sim:3:2,t);
    memo1.text:=t;


    {save the results to a text file}
    append(to_file_out);
    for b:=1 to 100 do writeln(to_file_out,current_sim,' ',b,' ',pop_size[b]);    {we save all simulations}
    if current_sim= n_sim then
      for b:=1 to 100 do writeln(to_file_out,'avg',' ',b,' ',FormatFloat('0.0', sum_pop_size[b]/current_sim)); {and the average of all simulations}
    CloseFile(to_file_out);

    application.processmessages;
  end;
end;

procedure Tdensity_dependent_Form.Exit_ButtonClick(Sender: TObject);
begin
  close;
end;

procedure Tdensity_dependent_Form.Abort_ButtonClick(Sender: TObject);
begin
  halt;
end;

end.

